@@ -1,7 +1,7 @@
 package Config::IOD::Document;
 
-our $DATE = '2015-05-05'; # DATE
-our $VERSION = '0.16'; # VERSION
+our $DATE = '2015-08-18'; # DATE
+our $VERSION = '0.17'; # VERSION
 
 use 5.010;
 use strict;
@@ -118,6 +118,12 @@ sub _discard_cache {
 
 sub dump {
     my $self = shift;
+    my $opts;
+    if (ref($_[0]) eq 'HASH') {
+        $opts = shift;
+    } else {
+        $opts = {};
+    }
 
     my $parser = $self->{_parser};
 
@@ -145,6 +151,9 @@ sub dump {
 
     for my $line (@{ $self->{_parsed} }) {
         $linum++;
+        next if defined($opts->{linum_start}) && $linum < $opts->{linum_start};
+        next if defined($opts->{linum_end}  ) && $linum > $opts->{linum_end};
+
         my $type = $line->[COL_TYPE];
         if ($type eq 'D') {
             my $directive = $line->[COL_D_DIRECTIVE];
@@ -232,6 +241,61 @@ sub _find_section {
     return @res;
 }
 
+sub each_section {
+    my $self = shift;
+    my $opts;
+    if (ref($_[0]) eq 'HASH') {
+        $opts = shift;
+    } else {
+        $opts = {};
+    }
+    my ($code) = @_;
+
+    my $parsed = $self->{_parsed};
+    my @linums = $self->_find_section({all=>1});
+    my %seen;
+    for my $linum (@linums) {
+        my $section = $parsed->[$linum-1][COL_S_SECTION];
+        next if $opts->{unique} && $seen{$section}++;
+
+        my $linum_end = $linum;
+        while (1) {
+            last if $linum_end >= @$parsed;
+            last if $parsed->[$linum_end][COL_TYPE] eq 'S';
+            $linum_end++;
+        }
+
+        $code->(
+            $self,
+            linum       => $linum,
+            linum_start => $linum,
+            linum_end   => $linum_end,
+            parsed      => $parsed->[$linum-1],
+            section     => $section,
+        );
+    }
+}
+
+sub list_sections {
+    my $self = shift;
+    my $opts;
+    if (ref($_[0]) eq 'HASH') {
+        $opts = shift;
+    } else {
+        $opts = {};
+    }
+
+    my @res;
+    $self->each_section(
+        $opts,
+        sub {
+            my ($self, %args) = @_;
+            push @res, $args{section};
+        }
+    );
+    @res;
+}
+
 sub _get_section_line_range {
     my $self = shift;
     my $opts;
@@ -608,7 +672,7 @@ Config::IOD::Document - Represent IOD document
 
 =head1 VERSION
 
-This document describes version 0.16 of Config::IOD::Document (from Perl distribution Config-IOD), released on 2015-05-05.
+This document describes version 0.17 of Config::IOD::Document (from Perl distribution Config-IOD), released on 2015-08-18.
 
 =head1 SYNOPSIS
 
@@ -724,12 +788,44 @@ occurrence.
 
 =back
 
-=head2 $doc->dump() => hoh
+=head2 $doc->dump([ \%opts ]) => hoh
 
 Return a hoh (hash of section names and hashes, where each of the second-level
 hash is of keys and values), Values will be decoded and merging will be done,
 but includes are not processed (even though C<include> directive is active).
 
+Options:
+
+=over
+
+=item * linum_start => int
+
+Only dump beginning from this line number.
+
+=item * linum_end => int
+
+Only dump until this line number.
+
+=back
+
+=head2 $doc->each_section([ \%opts , ] $code) => LIST
+
+Execute C<$code> for each section found in document, in order of occurrence.
+C<$code> will be called with arguments C<< ($self, %args) >> where C<%args> will
+contain these keys: C<section> (str, section name), C<linum> (int, line number,
+1-based), C<linum_start> (the same as C<linum>), C<linum_end> (int, line number
+of the last line of the section), C<parsed> (array, parsed line).
+
+Options:
+
+=over
+
+=item * unique => bool
+
+If set to 1, will only list the first occurence of each section.
+
+=back
+
 =head2 $doc->empty()
 
 Empty document.
@@ -812,6 +908,20 @@ Optional. Insert at this specific line number. Ignores C<top>.
 
 =back
 
+=head2 $doc->list_sections([ \%opts ]) => LIST
+
+List sections in the document, in order of occurrence.
+
+Options:
+
+=over
+
+=item * unique => bool
+
+If set to 1, will only list the first occurrence of each section.
+
+=back
+
 =head1 SEE ALSO
 
 L<Config::IOD>
